/*
 *		 Copyright (c) Nov 14 18:01:29  MTCC
 *	Author: Michael Thomas
 *	Module: account.js
 *	Created: Mon Nov 14 18:01:29 2011
 *	Abstract:
 *	   account/misc management for phresheez app
 */

/* Edit History: 
 */


var phzAccountSingleton = null;


function phzAccount (prefix, container) {
    this.curuser = '';
    this.curpass = '';
    this.curemail = ''; 
    this.asktotrack = false;
    this.largepics = false;
    this.trackingtimer = 10;
    this.batterytimer = 20;
    this.debug = false;
    this.candebug = false;
    this.precision = null;
    this.prefix = prefix;
    this.usedDefault = false;
    this.container = container;
    this.errorLog = '--error log empty--';
    this.protocol = document.location.protocol;
    this.platform = 'web';
    this.appAlertQueue = [];
    this.autoFilledUsername = false;	
    this.lastPage = this.displayHome;
    this.pagebg = 'style="background:white;  border-radius:6px; min-height: 360px;"';
    // domains with common prefixes will be displayed in this order
    this.domAC = new this.keyAutocomplete (['yahoo.com', 'gmail.com', 'hotmail.com', 'comcast.net', 'sbcglobal.net', 'aol.com', 'googlemail.com']);
    if (this.protocol == 'file:')
	this.protocol = 'https:';
    phzAccountSingleton = this;
}	


phzAccount.prototype.queueAppAlert = function (cmd) {
    try {
	queueAppAlert (cmd);
    } catch (e) {
	var state = this;
	var now = new Date ().getTime ();
	state.appAlertQueue [state.appAlertQueue.length] = { cmd:cmd, qtime:now};
	if (state.appAlertQueue.length == 1) {
	    document.location = cmd;
	    var fn = function () {
		state.appAlertQueue.shift ();	    
		if (state.appAlertQueue.length) {
		    document.location = state.appAlertQueue [0].cmd;
		    setTimeout (fn, 100);
		} 
	    };
	    setTimeout (fn, 100);
	} 
    }
};



phzAccount.prototype.reloadWidget = function () {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.reloadWidget ();
	} else {
	    this.queueAppAlert ('phz:reloadWidget');
	}
    } else {
	top.location.reload ();
    }
};

phzAccount.prototype.sendErrorLog = function () {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.sendErrorLog ();
	} else {
	    this.queueAppAlert ('phz:sendErrorLog');
	}
    } else {
	this.errorLog = '--error log empty--';
    }
};

phzAccount.prototype.deleteCrashLog = function () {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.deleteCrashLog ();
	} else {
	    this.queueAppAlert ('phz:deleteCrashLog');
	}
    } 
};

phzAccount.prototype.exitApp = function () {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.exitApp ();
	} else {
	    this.queueAppAlert ('phz:exitApp');
	}
    } 
};

phzAccount.prototype.flushPoints = function () {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.flushPoints ();
	} else {
	    this.queueAppAlert ('phz:flushPoints');
	}
    } 
};

phzAccount.prototype.popupYoutube = function (vid) {
    if (! isWidget ()) {
	window.open('http://www.youtube.com/embed/'+vid,'Demo', 'width=640,height=360');
    } else if (window.phz) {
	window.phz.popupYoutube (vid);
    } else {
	this.queueAppAlert ('phz:popupYoutube:'+vid);
    }
};

phzAccount.prototype.joined = function (uname, pass, joined) {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.joined ();
	} else {
	    this.queueAppAlert ('phz:joined');
	}
    } else
	this.reloadWidget ();
};

phzAccount.prototype.savePrefs = function () {
    if (isWidget ()) {
	// Note: Android 1.6 doesn't have JSON.*
	var sprefs = '{ "username":"'+this.curuser+'", "password":"'+this.curpass+'", "email":"'+this.curemail+'",';
	sprefs += '"asktotrack":'+this.asktotrack+', "largepics":'+this.largepics+', ';
	sprefs += '"debug":'+this.debug+', "passwordLocked":true,';
	sprefs += '"trackingtimer":'+this.trackingtimer+', "batterytimer":'+this.batterytimer+',';
	sprefs += '"precision":'+this.precision+'}';
	if (window.phz) {
	    window.phz.saveAccountInfo (sprefs);
	} else {
	    this.queueAppAlert ('phz:saveAccountInfo'+encodeURIComponent("\001")+sprefs);
	}
    } 
};


phzAccount.prototype.sendAccountInfo = function () {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.sendAccountInfo ();
	} else {
	    this.queueAppAlert ('phz:sendAccountInfo');
	}
    }
};

phzAccount.prototype.leaveApp = function () {
    if (isWidget ()) {
	if (! window.phz) {
	    try {
		queueAppAlert ("phz:leaveApp");
	    } catch (e) {};
	} else {
	    try {
		window.phz.leaveApp ();
	    } catch (e) {};
	}
    }
};


phzAccount.prototype.hasCredentials = function () {
    return ! ((this.usedDefault ||  ! this.curuser || ! this.curpass) && isWidget ());
};

phzAccount.prototype.topMenu = function (noalert) {
    var newc = '';
    newc += '<table width=320 cellpadding=0 cellspacing=0>';
    newc += '<tr>';
    var login = internalApp ? "Login" : "Prefs";
    newc += '<td onclick="'+this.prefix+'.displayLogin ()" class=topbuttonfirst>'+login;
    newc += '<td onclick="'+this.prefix+'.displayLearn ()" class=topbutton>Demo';
    var showmore = this.hasCredentials ();
    if (showmore || ! internalApp) {
	newc += '<td id='+this.prefix+'.topreload onclick="'+this.prefix+'.reloadWidget ()" class=topbutton>Reload';
	newc += '<td id='+this.prefix+'.topmore onclick="'+this.prefix+'.displayMore ()" class=topbutton>More';
    }
    newc += '<td onclick="'+this.prefix+'.displayJoin ()" class=topbuttonlast>Join';
    if (! noalert) {
	newc += '<tr><td colspan='+(showmore ? 5 : 3)+' align=center><div class=loadingBox id='+this.prefix+'.loading style="font-size:16px; font-weight:bold; color:red; visibility:hidden; width:300px; min-height:20px; text-align:center; margin-top:5px;"></div>';
    }
    newc += '</table>';
    return newc;
};

phzAccount.prototype.displayHome = function () {
    this.lastPage = this.displayHome;
    var newc = '';      
    newc += this.topMenu (true);
    var showstart = ! this.hasCredentials ();
    if (showstart) 
        newc += '<img src=imgs/unleash.jpg onclick="'+this.prefix+'.displayJoin ()">';
    else
        newc += '<img src=imgs/unleash.jpg>';
    var el = document.getElementById (this.container);
    this.reliableNewc (el, newc);
    el.style.display = 'block';
    el.style.background = 'white';
    el.style.height = '480px';			// XXX: hack!
};

phzAccount.prototype.displayJoin = function () {
    this.lastPage = this.displayJoin;
    this.autoFilledUsername = false;	
    this.foundAtSign = false;
    var newc = '';
    newc += this.topMenu ();
    newc += '<div '+this.pagebg+'>';
    newc += '<div style="text-align:center;">';
    //newc += '<table width=100% align=center><tr valign=middle style="font-size:18px;"><td>Join using<td><img src=imgs/facebook.jpg onclick="'+this.prefix+'.sendJoin (true)" class=clickable><td>or...</table>';
    newc += '</div>';

    newc += '<table width=100% cellpadding=2>';
    newc += '<tr><td><b>Email</b><td><input style="width:90%" type=email autocomplete=on id='+this.prefix+'.email value="'+this.curemail+'">';	
    newc += '<tr><td><b>Username</b><td><input style="width:90%" id='+this.prefix+'.username value="'+this.curuser+'">';
    if (this.usedDefault && this.curpass) {
        newc += "<tr><td colspan=3 style=\"font-size:14px\">We've created a secure password for you; Feel free to change it.";
    }
    newc += '<tr><td><b>Password</b><td><input style="width:90%" id='+this.prefix+'.pass1 type=password  value="'+this.curpass+'">';
    newc += '<tr><td><b>Retype Password</b><td><input style="width:90%" id='+this.prefix+'.pass2 type=password  value="'+this.curpass+'">';
    if (this.candebug) {
        if (this.debug)
            checked = 'checked=checked';
        else
            checked = '';
        newc += '<tr><td><b>Debug</b><td><input type=checkBox id='+this.prefix+'.debug '+checked+'>';
    }    
    newc += '</table>';

    newc += '<table width=100% style="margin-top:20px">';
    newc += '<tr align=right>';
    newc += '<td><button onclick="'+this.prefix+'.sendJoin (false)" class=phzButton  style="margin-right:8px">Join</button>';
    newc += '</table>';
    newc += '</div>';

    var state = this;
    var el = document.getElementById (this.container);
    this.reliableNewc (el, newc, function () {
			   //state.setUsernameListener ();
			   state.setEmailListener ();
			       });
};

phzAccount.prototype.displayLogin = function () {
    this.lastPage = this.displayLogin;
    var newc = '';
    newc += this.topMenu ();
    var user = this.usedDefault ? '' : this.curuser;
    var pass = this.usedDefault ? '' : this.curpass;
    newc += '<div '+this.pagebg+'>';
    newc += '<table width=100% cellspacing=2>';
    newc += '<tr><td><b>Username</b><td><input style="width:90%" id='+this.prefix+'.username  value="'+user+'">';
    newc += '<tr><td><b>Password</b><td><input style="width:90%" id='+this.prefix+'.pass1 type=password  value="'+pass+'">';
    var checked;
    if (this.asktotrack)
	checked = 'checked=checked';
    else
	checked = '';
    newc += '<tr><td><b>Ask to Track?</b><td><input type=checkBox id='+this.prefix+'.asktotrack '+checked+'>';
    if (this.candebug) {
        if (this.debug)
            checked = 'checked=checked';
        else
            checked = '';
        newc += '<tr><td><b>Debug</b><td><input type=checkBox id='+this.prefix+'.debug '+checked+'>';
    }
    if (this.hasAdvanced ())
        newc += '<tr><td colspan=2 align=center><input type=button onclick="'+this.prefix+'.displayAdvanced ()" value="Advanced" style="width:120px;height:30px;border-radius:10px;">';
    newc += '</table>';
    newc += '<table width=320 style="margin-top:20px">';
    newc += '<tr>';
    newc += '<td align=left><button onclick="'+this.prefix+'.displayForgot ()" class=phzButton style="width:105px; margin-left:8px">I Forgot It</button>';
    var login = internalApp ? "Login" : "Save";
    newc += '<td align=right><button onclick="'+this.prefix+'.sendLogin ()" class=phzButton  style="margin-right:8px">'+login+'</button>';
    if (! isWidget ()) {
	newc += '<tr><td colspan=3><h3>Note: this will not change your in-app settings</h3>';
    }
    newc += '</table>';			 
    newc += '</div>';
    var el = document.getElementById (this.container);
    this.reliableNewc (el, newc);
};

phzAccount.prototype.displayForgot = function () {
    this.lastPage = this.displayLogin;
    var html = '';
    var user = this.usedDefault ? '' : this.curuser;
    var checked = '';
    var login;
    if (user) {
	checked = 'checked=checked';
    }
    html += this.topMenu ();
    html += '<div '+this.pagebg+'>';
    html += '<table width=100%><tr><td><b>Username or Email</b><td><input id='+this.prefix+'.uname value="'+user+'" size=16 style="width:100%">';
    html += '<tr><td valign=top><input type=radio name=forgot id='+this.prefix+'.forgot value=1 '+checked+'>';
    html += '<td> I don\'t know my password';
    html += '<tr><td valign=top><input type=radio name=forgot id='+this.prefix+'.forgotboth value=1>';
    html += '<td> I don\'t know my username and password: enter your email in the username field';
    html += "<tr><td colspan=2><br>";
    html += "<tr><td colspan=2><strong>Your login information will be mailed to your registered email address.</strong>";
    html += "<tr><td colspan=2><br>";
    html += '<tr valign=center><td align=left><button onclick="'+this.prefix+'.displayLogin ()" class=phzButton style="width:100px;margin-left:8px">Cancel</button>';
    html += '<td align=right><button onclick="'+this.prefix+'.sendForgot ()" class=phzButton style="margin-right:8px">Send</button>';
    html += '</table>';
    html += '</div>';
    var el = document.getElementById (this.container);
    this.reliableNewc (el, html);
};


phzAccount.prototype.sendForgot = function () {
    var url = baseurl + 'login.php?mode=ajax';
    var el = document.getElementById (this.prefix+'.uname');
    var user = el.value;
    url += '&uname='+encodeURIComponent (user);
    var el = document.getElementById (this.prefix+'.forgot');
    if (el.checked) {
	url += '&forgot='+(el.checked ? '1' : '0');
    }
    var el = document.getElementById (this.prefix+'.forgotboth');
    if (el.checked) {
	url += '&forgotboth=1';
	if (user.indexOf ('@') < 0) {
	    this.loadingAlert ("Enter a valid email address");
	    return;
	}
    }
    var state = this;
    this.fetchServer ("GET", url, 
		      function (resp, param, sts) {
			  var r = new state.phzResp (resp);
			  if (sts < 300 && sts != 0) {
			      if (r.resp < 300) {
				  state.displayLogin ();
				  state.loadingAlert ("Your login information has been sent to your email address");

			      } else if (r.resp >= 500) {
				  state.loadingAlert ("Error with login information: "+r.comment);
			      } else if (r.resp == 402) {
				  state.loadingAlert ("A network error occurred. Try again later");
			      } else {
				  state.loadingAlert ("Internal error "+r.resp+' '+resp+" httpsts="+sts+" u="+url);
			      }
			  } else {
			      state.loadingAlert ("Can't connect to network right now; try again later");
			  }
		      });

};

phzAccount.prototype.setUsernameListener = function () {
    var el = document.getElementById (this.prefix+'.username');
    if (! el)
	return;
    var state = this;
    var lfn = function () {
	var el2 = document.getElementById (state.prefix+'.email');
	if (el2 && ! el.value && ! state.autoFilledUsername) {
	    var idx = el2.value.indexOf ('@');
	    if (idx >= 0) {
		el.value = el2.value.substr (0, idx);
		state.autoFilledUsername = true;
	    }
	}
    };
    if (document.addEventListener)
	el.addEventListener ('focus', lfn, true);
};

phzAccount.prototype.keyAutocomplete = function (completions) {
    this.completions = completions;
};

phzAccount.prototype.keyAutocomplete.prototype.run = function (key, el, idx) {
    var val = idx ? el.value.substr (idx) : el.value;
    if (val.length > 0 && key != 8 && key != 46) {
	for (var i in this.completions) {
	    var c = this.completions [i];
	    if (c.indexOf (val) == 0) {
		el.value = el.value.substr (0, idx)+c;
		el.setSelectionRange(idx+val.length, el.value.length);
		break;
	    }
	}
    }
};

phzAccount.prototype.setEmailListener = function () {
    var el = document.getElementById (this.prefix+'.email');
    if (! el)
	return;
    var state = this;
    var lfn = function (ev) {
	var keyCode;
	if (window.event) {
	    ev = window.event;
	    keyCode = ev.keyCode;
	} else {
	    if (ev.keyCode == ev.which) 
		keyCode = null;
	    else if (ev.charCode != ev.which)      
		keyCode = void 0;
	    else 	// else a char - set it
		keyCode = ev.which;
	    keyCode = ev.keyCode;
	}
	if (keyCode == 50) {
	    state.foundAtSign = true;
	    var el2 = document.getElementById (state.prefix+'.username');
	    state.checkUserTaken (el2.value, el2);
	} else {
	    var idx;
	    if ((idx = el.value.indexOf ('@')) < 0) {
		state.foundAtSign = false;
	    } 
	    if (state.foundAtSign) {
		state.domAC.run (keyCode, el, idx+1);
	    } else {
		if (! state.curuser && el.value.indexOf ('@') < 0) {
		    var el2 = document.getElementById (state.prefix+'.username');
		    el2.value = el.value;
		}
	    }
	}

    };
    if (document.addEventListener)
	el.addEventListener ('keyup', lfn, true);
};

phzAccount.prototype.checkUserTaken = function (uname, el) {
    var debug = document.getElementById (this.prefix+".debug");
    if (debug)
	this.debug = debug.checked ? true : false;
    else
	this.debug = false;
    this.setBaseurl ();
    var url = this.sbaseurl + "adduser.php?mode=checkavail"+
    "&uname="+encodeURIComponent(uname);
    url += "&cb="+Math.random ();
    var state = this;
    this.fetchServer ("GET", url, 
		      function (resp, param, sts) {
			  var r = new state.phzResp (resp);
			  if (r.resp == 201 && el.value == uname) {		// taken, and then username is still the same?
			      var avail = state.jsonEval (r.body);
			      state.loadingAlert (uname+" is taken. suggesting "+avail.available);
			      var el2 = document.getElementById (state.prefix+'.username');
			      el2.value = avail.available;
			  }
		      });
};

phzAccount.prototype.jsonEval = function (str) {
    try {
	return JSON.parse (str);
    } catch (e) {
	return eval ('('+str+')');
    }
};


phzAccount.prototype.displayLearn = function () {
    this.lastPage = this.displayLearn;
    //this.popupYoutube ('g3cHxOM0NY0');
    this.popupYoutube ('Xahl2N5voRU');
    return;
    var newc = '';
    newc += '<iframe height="480" width="274" src="http://www.youtube.com/embed/g3cHxOM0NY0" frameborder="0" allowfullscreen></iframe>';
    var el = document.getElementById (this.container);
    newc += '<br><button onclick="'+this.prefix+'.displayHome ()" class=phzButton>Done</button>';
    this.reliableNewc (el, newc);
};

phzAccount.prototype.hasAdvanced = function () {
    return true;
    if (isWidget ())
	return true;
    else
	return false;
};

phzAccount.prototype.displayAdvanced = function () {
    var newc = '';
    newc += '<div '+this.pagebg+'>';
    newc += '<h3></h3>';
    newc += '<table width=100% cellpadding=10>';
    if (isAndroid ()) {
        newc += '<tr id="'+this.prefix+'.precisionrow" style="display:'+(this.precision != null ? 'table-row':'none')+'">';
        newc += '<td><b>Track Precision</b><td><select id='+this.prefix+'.precision>';
        var selected;
        selected = this.precision == 0 || this.precision == 1 || this.precision == 2 ? 'selected=selected' : '';
        newc += '<option value=1 '+selected+'>Normal';
        selected = this.precision == 3 ? 'selected=selected' : '';
        newc += '<option value=3 '+selected+'>Battery Saver';
        //selected = this.precision == 2 ? 'selected=selected' : '';
        //newc += '<option value=2 '+selected+'>High Resolution';
        newc += '</select>';
        var checked;
        if (this.largepics)
            checked = 'checked=checked';
        else
            checked = '';
        newc += '<tr><td><b>HighRes Photos</b><td><input type=checkBox id='+this.prefix+'.largepics '+checked+'>';
    } else {
        newc += '<input type=hidden id='+this.prefix+'.largepics value=0>';
        newc += '<input type=hidden id='+this.prefix+'.precision value=1>';
    }
    newc += '<tr><td><b>Stop Tracking After This Amount of Time</b>';
    newc += '<td>' + htmlselect (this.prefix+".trackingtimer", {0:"Off",
					 1:"1 Hour", 2:"2 Hours", 3:"3 Hours", 4:"4 Hours", 5:"5 Hours", 6:"6 Hours",
					 7:"7 Hours",8:"8 Hours", 9:"9 Hours", 10:"10 Hours", 11:"11 Hours", 12:"12 Hours",
					 13:"13 Hours",14:"14 Hours", 15:"15 Hours", 16:"16 Hours"}, this.trackingtimer, '');

	// XXX this is somewhat problematic on android, so just leave it out until it gets support
    if (isIphone ()) {
        newc += '<tr><td><b>Stop Tracking When Battery Reaches This Level</b>';
        newc += '<td>' + htmlselect (this.prefix+".batterytimer", {0:"Off",
                             10:"10%", 20:"20%", 30:"30%", 40:"40%", 50:"50%", 60:"60%",
                             70:"70%",80:"80%", 90:"90%"}, this.batterytimer, '');
    }

    var to = this.lastPage == this.displayJoin ? ".displayJoin ()" : ".displayLogin ()";
    newc += '<tr><td align=left><button onclick="'+(this.prefix+to)+'" class=phzButton style="width:100px;margin-left:8px">Cancel</button>';
    newc += '<td align=right><button onclick="'+this.prefix+'.saveAdvanced ()" class=phzButton style="margin-left:8px">Save</button>';
    newc += '</table>';
    newc += '</div>';

    var el = document.getElementById (this.container);
    this.reliableNewc (el, newc);
};

phzAccount.prototype.saveAdvanced = function () {	
    var el;
    el = document.getElementById (this.prefix+".largepics");
    if (el)
	this.largepics = el.checked ? true : false;
    el = document.getElementById (this.prefix+".precision");    
    if (el)
	this.precision = el.value;
    el = document.getElementById (this.prefix+".trackingtimerbutton");    
    if (el)
	this.trackingtimer = el.value;
    el = document.getElementById (this.prefix+".batterytimerbutton");    
    if (el)
	this.batterytimer = el.value;
    this.lastPage ();
};

phzAccount.prototype.displayMore = function () {
    this.lastPage = this.displayMore;
    var newc = '';
    newc += this.topMenu ();
    newc += '<div style="height:20px; width:100%"></div>';
    newc += '<table width=100% cellpadding=10>';
    newc += '<tr align=center><td><button onclick="'+this.prefix+'.sendPoints ()" class=phzButton style="width:260px">Send Points/Images Now</button>';
    newc += '<tr align=center><td><button onclick="'+this.prefix+'.displayGallery ()" class=phzButton style="width:260px">Upload Photos</button>';
    newc += '<tr align=center><td><button onclick="'+this.prefix+'.displayErrorlog ()" class=phzButton style="width:260px">App Log</button>';
    newc += '<tr align=center><td><button onclick="'+this.prefix+'.displayProblem ()" class=phzButton style="width:260px">Request Support</button>';
    if (isAndroid ())
	newc += '<tr align=center><td><button onclick="'+this.prefix+'.appExit ()" class=phzButton style="width:260px">Exit</button>';
    newc += '</table>';
    var el = document.getElementById (this.container);
    this.reliableNewc (el, newc);
};

phzAccount.prototype.loadingAlert = function (title) {
    if (this.loadingTmo) {
        clearTimeout (this.loadingTmo);
        this.loadingTmo = null;
    }
    var el = document.getElementById (this.prefix+".loading");
    el.style.visibility = 'visible';
    this.reliableNewc (el, title);
    var state = this;
    this.loadingTmo = setTimeout (function () {
				      el.style.visibility = 'hidden';	
				      state.loadingTmo = null;
				  }, 5000);
};

phzAccount.prototype.checkInput = function (joinmode, pass2) {
    if (! this.curuser || ! this.curpass) {
	if (joinmode)
	    this.loadingAlert ("Fill in email and password");
	else
	    this.loadingAlert ("Fill in username and password");
        return false;
    }
    if (this.curpass.length < 6) {
        this.loadingAlert ("Passwords must be at least 6 characters long");
        return false;
    }
    if (this.curuser.indexOf ('<') >= 0 ||this.curuser.indexOf ('>') >= 0 ||
	    this.curuser.indexOf ('\'') >= 0 || this.curuser.indexOf ('"') >= 0 ||	
	    this.curuser.indexOf (' ') >= 0 || this.curuser.indexOf ('\t') >= 0) {
        this.loadingAlert ("Usernames may not contain <>'\" space or tab");
        return false;
    }
    if (joinmode) {
        if (this.curpass != pass2) {
            this.loadingAlert ("Passwords don't match");
            return false;
        }
        if (this.curemail.indexOf ('@') <= 0) {
            this.loadingAlert ("Enter proper email address");
            return false;
        }
    } else {
        // Nothing
    }
    return true;
};

phzAccount.prototype.sendLogin = function () {
    this.curuser = document.getElementById (this.prefix+".username").value;
    this.curpass = document.getElementById (this.prefix+".pass1").value;
    this.asktotrack = document.getElementById (this.prefix+".asktotrack").checked ? true : false;
    var debug = document.getElementById (this.prefix+".debug");
    if (debug)
	this.debug = debug.checked ? true : false;
    else
	this.debug = false;
    this.setBaseurl ();
    if (! this.checkInput (false))
	return;
    var url = this.sbaseurl + "adduser.php?mode=test"+
    "&pass="+encodeURIComponent(this.curpass)+
    "&uname="+encodeURIComponent(this.curuser);
    var state = this;
    this.loadingAlert ("Verifying username/password...");
    this.savePrefs ();
    this.fetchServer ("GET", url, 
		      function (resp, param, sts) {
			  var r = new state.phzResp (resp);
			  if (sts < 300 && sts != 0) {
			      if (r.resp < 300) {
				  state.loadingAlert ("Logging into app...");
				  state.reloadWidget ();
				  setTimeout (function () {
						  if (isWidget ())
						      state.loadingAlert ("Oops... can't log in");
						  else
						      state.displayHome ();
					      }, 2000);
			      } else if (r.resp >= 500) {
				  state.loadingAlert ("Error logging in: "+r.comment);
			      } else if (r.resp == 402) {
				  state.loadingAlert ("A network error occurred. Try again later");
			      } else {
				  state.loadingAlert ("Internal error "+r.resp+' '+resp+" httpsts="+sts+" u="+url);
			      }
			  } else {
			      state.loadingAlert ("Can't connect to network right now; try again later");
			  }
		      });
};

phzAccount.prototype.sendJoin = function (withfacebook) {
    this.curuser = document.getElementById (this.prefix+".username").value;
    this.curpass = document.getElementById (this.prefix+".pass1").value;
    this.curemail = document.getElementById (this.prefix+".email").value;
    var pass2 = document.getElementById (this.prefix+".pass2").value;
    var debug = document.getElementById (this.prefix+".debug");
    if (debug)
        this.debug = debug.checked ? true : false;
    else
        this.debug = false;
    this.setBaseurl ();
    if (withfacebook) {
        if (this.usedDefault) {
            this.curuser = '';
            this.curpass = '';
            this.curemail = '';
        }
        this.savePrefs ();	
        this.leaveApp ();
        this.loadingAlert ("Sending Facebook Join Request...");
        var state = this;
        setTimeout (function () {
                var url = state.sbaseurl+"fbapp.php?mode=join&platform="+encodeURIComponent(state.platform)+
                    "&app="+encodeURIComponent(state.appName);
                top.location.href = url;
                }, 300);
        return;
    }
    if (! this.checkInput (true, pass2))
        return;
    this.loadingAlert ("Sending Join Request...");
    var url = this.sbaseurl + "adduser.php?mode=ajax"+
    "&pass1="+encodeURIComponent(this.curpass)+
    "&pass2="+encodeURIComponent(this.curpass)+
    "&uname="+encodeURIComponent(this.curuser)+
    "&email="+encodeURIComponent(this.curemail)+
    "&platform="+encodeURIComponent(this.platform)+
    "&app="+encodeURIComponent(this.appName);
    var state = this;
    this.savePrefs ();
    this.fetchServer ("GET", url, 
		      function (resp, param, sts) {
			  var r = new state.phzResp (resp);
			  if (sts < 300 && sts != 0) {
			      if (r.resp < 300) {
				  state.loadingAlert ("Join succeeded,  logging into app...");
				  state.joined ();
				  setTimeout (function () {
						  if (isWidget ())
						      state.loadingAlert ("Oops... can't log in");
						  else
						      state.displayHome ();
					      }, 2000);
			      } else if (r.resp >= 500) {
				  state.loadingAlert ("Error logging in: "+r.comment);
			      } else {
				  state.loadingAlert ("Internal error "+r.resp+' '+resp);
			      }
			  } else {
			      state.loadingAlert ("Can't connect to the network right now; try again later");
			  }
		      });
};


phzAccount.prototype.sendPoints = function () {
    this.displayMore ();
    this.flushPoints ();
    this.loadingAlert ("Points scheduled to be sent");
};

phzAccount.prototype.sendProblem = function () {
    var url = this.sbaseurl + 'problem.php';
    var postdata = 'mode=add&platform='+this.platform;
    postdata += '&category='+encodeURIComponent (document.getElementById (this.prefix+'.category').value);
    postdata += '&body='+encodeURIComponent (document.getElementById (this.prefix+'.body').value);
    postdata += "\n---------Log Contents-------------\n\n"+this.errorLog;
    var state = this;
    this.fetchServer ("POST", url, function (resp, param, sts) {
			  var r = new state.phzResp (resp);
			  if (sts < 300) {
			      if (r.resp < 300) {
				  state.displayMore ();
				  state.loadingAlert ("Report sent successfully");
				  state.deleteCrashLog ();
			      } else if (r.resp >= 500) {
				  state.displayMore ();
				  state.loadingAlert ("Can't send report: "+r.comment);
			      } else {
				  state.displayMore ();
				  state.loadingAlert ("Internal error "+r.resp+' '+resp);
			      }
			  } else {
			      state.displayMore ();
			      state.loadingAlert ("Can't connect to the network right now<br>try again later");
			  }			  
		      }, null, postdata);
    this.loadingAlert ("Sending support request...");
};

phzAccount.prototype.appExit = function () {
    this.displayHome ();
    this.loadingAlert ("Exiting...");
    this.exitApp ();
};

phzAccount.prototype.displayGallery = function () {
    if (isWidget ()) {
	if (window.phz) {
	    window.phz.displayGallery ();
	} else {
	    this.queueAppAlert ('phz:displayGallery');
	}
    } else {
	this.loadingAlert ("Only supported in app...");
    }
};



phzAccount.prototype.displayErrorlog = function () {
    var newc = '';
    newc += this.topMenu ();
    newc += '<div '+this.pagebg+'>';
    newc += '<h3><button class=phzButton onclick="'+this.prefix+'.displayErrorlog ()">App Log</button></h3>';
    newc += '<div id="'+this.prefix+'.errlog">'+this.errorLog+'</div>';
    newc += '</div>';
    var el = document.getElementById (this.container);
    this.reliableNewc (el, newc);
    this.sendErrorLog ();
};

phzAccount.prototype.displayProblem = function (category) {
    var newc = '';
    this.sendErrorLog ();
    newc += this.topMenu ();
    newc += '<div '+this.pagebg+'>';    
    newc += "<h3>Tell us what's up... we listen!</h3>";
    newc += '<div style="text-align:center">';
    newc += '<select id="'+this.prefix+'.category">';
    newc += '<option '+(category == 0 ? 'selected=selected' : '')+'>General';
    newc += '<option '+(category == 1 ? 'selected=selected' : '')+'>Network';
    newc += '<option '+(category == 2 ? 'selected=selected' : '')+'>UI';
    newc += '<option '+(category == 3 ? 'selected=selected' : '')+'>GPS';
    newc += '<option '+(category == 4 ? 'selected=selected' : '')+'>Payment';
    newc += '</select><br>';
    newc += '<textarea id="'+this.prefix+'.body" cols=30 rows=10></textarea>';
    newc += '<br><button class=phzButton onclick="'+this.prefix+'.sendProblem ()">Send</button>';
    newc += '</div>';
    newc += '</div>';
    var el = document.getElementById (this.container);
    this.reliableNewc (el, newc);
};


phzAccount.prototype.fetchServer = function (method, query, fn, param, postdata) {
    var req;
    try {
	// Firefox, Opera 8.0+, Safari, IE7+
	req = new XMLHttpRequest(); // xmlHttp is now a XMLHttpRequest.
    } catch (e) {
	// Internet Explorer
	try {
	    req=new ActiveXObject("Msxml2.XMLHTTP");
	} catch (e) {
	    req=new ActiveXObject("Microsoft.XMLHTTP");
	}
    }
    req.onreadystatechange = function() {
	if (req.readyState == 4) {
	    if (req.status >= 200 || ! req.status) {
		fn (req.responseText, param, req.status);
	    }
	}
    };
    req.open(method, query); // .open(RequestType, Source);
    if (method == 'POST')
	req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    req.send(postdata); 
};

phzAccount.prototype.phzResp = function (resp) {
    this.resp = 402;
    this.comment = 'unexpected server error';
    this.body = '';
    var i = resp.indexOf ('\n');
    var r = resp.indexOf (' ');
    if (resp.length == 0 || i < 0 || r < 0) {
	return;	
    }
    this.resp = resp.substr (0, r);
    if (isNaN (this.resp)) {
	this.resp = 402;
	return;
    }
    this.comment = resp.substr (r+1, i-r-1);
    this.body = resp.substr (i+1, resp.length);
};

var internalApp = false;

try {
    if (sprintf)
	internalApp = false;
    else
	internalApp = true;
} catch (e) {
    internalApp = true;
}

if (internalApp) {

    loadProgress = function (curlen, totlen) {
	var state = phzAccountSingleton;
	var el = document.getElementById (state.prefix+'.loadProgress');
	if (el) 
	    state.reliableNewc (el, curlen + '/'+totlen);		
    };

    isWidget = function () {	
	return true;
    };

    isAndroid = function () {	
	return isWidget () && window.phz;
    };

    isIphone = function () {	
	return isWidget () && ! window.phz;
    };


    spinner = function () {
	//function spinner () {
	this.curoff = 123;
	this.id = "__spinner."+Math.random ();
	this.tmo = null;
    };

    spinner.prototype.step = function () {
	var el = document.getElementById (this.id);
	if (el == null)
	    return;
	var state = this;
	this.tmo = setTimeout (function () {
				   state.step (); } , 80);
	el.style.backgroundPosition = -this.curoff+"px 0px";
	this.curoff -= 1;
	if (this.curoff < 0)
	    this.curoff = 123;
    };

    spinner.prototype.stop = function () {
	if (this.tmo) {	
	    clearTimeout (this.tmo);
	    this.tmo = null;
	}
	var el = document.getElementById (this.id);
	el.style.visibility = 'hidden';
    };


    spinner.prototype.toHTML = function () {
	return '<img id="'+this.id+'" src=imgs/percentImage.png style="background: white url(imgs/indeterminate.jpg) top left no-repeat">';
    };

    htmlselect = function (name, arr, selected, code) {
	var out;
	if (code != null) 
	    out = "<select style=\"width:100%\" id=\""+name+"button\" name=\""+name+"\" onchange=\""+code+"\">";
	else
	    out = "<select style=\"width:100%\" id=\""+name+"button\" name=\""+name+"\">";
	var k, v;
	for (k in arr) {
	    v = arr [k];
	    if (k == selected)
		out += "<option value=\""+k+"\" selected=selected>"+v;
	    else
		out += "<option value=\""+k+"\">"+v;
	}
	out += "</select>";
	return out;
    };


}

phzAccount.prototype.setBaseurl = function () {
    this.sbaseurl = this.protocol+'//phresheez.com/site/';
    if (this.debug)
	this.sbaseurl = this.protocol+'//mtcc.com/site/';     
    else if (this.appName == 'Fresheez')
	this.sbaseurl = this.protocol+'//fresheez.com/site/';     
};

phzAccount.prototype.reliableNewc = function (el, content, oncomplete) {
    if (el.phzNewcTmo) {
	clearTimeout (el.phzNewcTmo);
	el.phzNewcTmo = null;
    }
    el.innerHTML = content;
    if (content && ! el.innerHTML) {
	el.phzNewcRetry = 0;
	var fn = function () {
	    el.innerHTML = content;
	    if (el.innerHTML || ++el.phzNewcRetry > 50) {
		el.phzNewcTmo = null;
		if (oncomplete)
		    oncomplete ();
		return;	    
	    }
	    el.phzNewcTmo = setTimeout (fn, 10);
	};
	setTimeout (fn, 10);
    } else if (oncomplete)
	oncomplete ();
    return el;
};



// app API call interface routines

function accountInfo (info) {
    var state = phzAccountSingleton;
    state.curuser = info.username;
    state.curpass = info.pass;
    state.curemail = info.email;
    state.asktotrack = info.asktotrack;
    state.largepics = info.largepics;
    state.batterytimer = info.batterytimer;
    state.trackingtimer = info.trackingtimer;
    state.debug = info.debug;
    state.candebug = info.candebug;
    state.appName = info.appName;
    state.usedDefault = info.usedDefault;
    state.precision = info.precision;
    state.platform = info.platform;
    state.setBaseurl ();
    var el;
    if ((el = document.getElementById (state.prefix+'.user')))
	el.value = state.curuser;
    if ((el = document.getElementById (state.prefix+'.pass')))
	el.value = state.curpass;
    if ((el = document.getElementById (state.prefix+'.pass1')))
	el.value = state.curpass;
    if ((el = document.getElementById (state.prefix+'.pass2')))
	el.value = state.curpass;
    if ((el = document.getElementById (state.prefix+'.email')))
	el.value = state.curemail;
    if ((el = document.getElementById (state.prefix+'.asktotrack')))
	el.checked = state.asktotrack;
    if ((el = document.getElementById (state.prefix+'.largepics')))
	el.checked = state.largepics;
    if ((el = document.getElementById (state.prefix+'.trackingtimerbutton')))
	el.value = state.trackingtimer;
    if ((el = document.getElementById (state.prefix+'.batterytimerbutton')))
	el.value = state.batterytimer;
    if ((el = document.getElementById (state.prefix+'.debug'))) {
	el.checked = state.debug;
    }
    if ((el = document.getElementById (state.prefix+'.precision'))) {
	el.value = state.precision;
    }
    if ((el = document.getElementById (state.prefix+'.precisionrow'))) {
	if (state.precision != null)
	    el.style.display = 'table-row';
	else
	    el.style.display = 'none';
    }
    var showstart = ! state.hasCredentials ();
    el = document.getElementById (state.prefix+".homeStart");
    if (el)
	el.style.display = showstart ? 'block' : 'none';
    return '200 ok\n';
};

function displayAccount () {
    var state = phzAccountSingleton;
    state.displayLogin ();
    return '200 ok\n';
}

function errorLogContents (log) {
    var state = phzAccountSingleton;
    state.errorLog = log;
    log = log.replace (/</g, "&lt;");
    log = log.replace (/>/g, "&gt;");
    log = log.replace (/\n/g, "<br>");
    var el = document.getElementById (state.prefix+".errlog");
    if (el)
	state.reliableNewc (el, log);
    return '200 ok\n';
}

function displayProblem (category) {
    var state = phzAccountSingleton;
    if (! internalApp) {
	setPage (8);
    }
    state.displayProblem (category);
}

if (internalApp) {

    backbutton = function () {
	var state = phzAccountSingleton;
	state.displayHome ();
	return '200 ok\n';
    };

}



